/*
 * AFS_API.c
 *
 *  Created on: Mar 4, 2014
 *      Author: Ian Cathey
 */

 #include <msp430.h>
 #include "AFS_API.h"
#include "usci_uart.h"

 unsigned int enabledChannels = 0x00;
 unsigned int numChannels = 0;
 unsigned int enabledAlarms = 0x00;
 unsigned int alarm = 0x00;
 unsigned int threshold = 0;									// this is the threshold below which an ADC reading will cause an alarm
 int sampleAverage[MAX_NUM_CHANNELS];							// average value of ADC_NUM_SAMPLES samples from ADC
 unsigned int numFails[MAX_NUM_CHANNELS];						// number of consecutive airflow fails
 unsigned int sampleIndex;										// ADC Value loop index

 unsigned int lastResult;

 unsigned int ADCValue[MAX_NUM_CHANNELS][ADC_NUM_SAMPLES];		// Measured ADC values

 /* This function does some basic configuration of the ADC10 on MSP430G2xxx series controllers */
 void AFS_configureADC10()
 {

	ADC10CTL0 &= ~(ENC+ADC10IE);				// Turn ADC off to change settings
	ADC10CTL0 |= ADC10SHT_2 + ADC10IE;					// Sample&Hold time 64 clks, enable interrupts
	ADC10CTL0 |= ENC + ADC10ON;							// Turn ADC back on
 }

 /* This function enables an individual analog input to the ADC */
 /* The user still needs to manually configure the GPIOs as inputs */
 /* TODO: Make return codes for Success, Illegal Channel, and NoAction (channel already enabled) instead of returning nothing */
 void AFS_enableChannel(int channel)
 {
	if ((channel < 0) | (channel > 7)) return;			// Make sure we only use legal channel values
	if ((0x01 << channel) & enabledChannels) return;	// If the channel is already enabled, do nothing
	else
	{
		enabledChannels |= (0x01 << channel);			// Mark this channel as enabled
		numChannels++;									// Add this channel to our total number of channels enabled
		ADC10CTL0 &= ~(ENC + ADC10IE);			// Turn ADC off to change settings
		ADC10AE0 |= (0x01 << channel);					// Enable port pins as analog inputs
		ADC10CTL0 |= ENC + ADC10IE;			// Turn ADC back on
	}
	return;
 }

 /* This function disables an individual analog input to the ADC */
 /* The user still needs to manually configure the GPIOs as inputs */
  void AFS_disableChannel(int channel)
 {
	if ((channel < 0) | (channel > 7)) return;			// Make sure we only use legal channel values
	if (!((0x01 << channel) & enabledChannels)) return;	// If the channel is already disabled, do nothing
	else
	{
		enabledChannels &= ~(0x01 << channel);			// Mark this channel as enabled
		numChannels--;									// Add this channel to our total number of channels enabled
		ADC10CTL0 &= ~(ENC+ADC10IE);					// Turn ADC off to change settings
		ADC10AE0 &= ~(0x01 << channel);					// Enable port pins as analog inputs
		ADC10CTL0 |= ENC + ADC10IE;			// Turn ADC back on
	}
	return;
 }

 /* This function returns a byte where each bit represents a channel */
 unsigned int AFS_returnEnabledChannels()
 {
	return enabledChannels;
 }

 /* This function returns the total number of channels currently enabled */
 unsigned int AFS_returnNumberChannels()
 {
	return numChannels;
 }

 /* This function returns a non-zero result if the channel being checked is enabled */
 unsigned int AFS_returnEnabledStatus(int channel)
 {
	if ((channel < 0) | (channel > 7)) return 0;			// Make sure we only check legal channel values
	return ((0x01 << channel) & enabledChannels);		// Returns a '1' if channel is already enabled, '0' otherwise
 }

 /* This function will set the level at which we interpret a signal level as failing or unfailing	*/
 /* compared to the quiescent value 																*/
 void AFS_setFailLevel(int level)
 {
	 if ((level < 0) | (level > 1024)) return;
	 threshold = level;
	 return;
 }

 /* This function enables the alarms of all enabled channels */
 void AFS_enableAlarmAll()
 {
	enabledAlarms = enabledChannels;
	return;
 }

 /* This function disables the alarms of all channels */
 void AFS_disableAlarmAll()
 {
	enabledAlarms = 0x00;
	AFS_clearAlarmAll();								// Clear any raised alarms
	return;
 }

 /* This function enables the alarm of an individual channel, provided it is enabled */
 void AFS_enableAlarm(int channel)
 {
	if ((channel < 0) | (channel > 7)) return;			// Make sure we only use legal channel values
	if (!((0x01 << channel) & enabledChannels)) return;	// If the channel is disabled, do nothing
	enabledAlarms |= (0x01 << channel);					// Enable the alarm for this channel
	return;
 }

 /* This function disables the alarm of an individual channel */
 void AFS_disableAlarm(int channel)
 {
	if ((channel < 0) | (channel > 7)) return;			// Make sure we only use legal channel values
	enabledAlarms &= ~(0x01 << channel);				// Disable the alarm for this channel
	AFS_clearAlarm(channel);							// Clear the alarm
	return;
 }

 /* This function returns a byte where each bit represents a channel */
 unsigned int AFS_returnEnabledAlarms()
 {
	return enabledAlarms;
 }

 /* This function returns a non-zero result if the alarm being checked is enabled */
 unsigned int AFS_returnEnabledAlarmStatus(int channel)
 {
	if ((channel < 0) | (channel > 7)) return 0;		// Make sure we only check legal channel values
	return ((0x01 << channel) & enabledAlarms);			// Returns a '1' if channel is already enabled, '0' otherwise
 }

 /* This function returns a non-zero result if the channel being checked has raised an alarm */
 unsigned int AFS_returnAlarmStatus(int channel)
 {
	if ((channel < 0) | (channel > 7)) return 0;		// Make sure we only check legal channel values
	return ((0x01 << channel) & alarm);					// returns a '1' if alarm for specified channel has been raised
 }

 unsigned int AFS_returnAlarmStatusAll()
 {
	return alarm;										// Each bit represents the alarm for each channel
 }

 /* This function clears the alarm of a specific channel */
 void AFS_clearAlarm(int channel)
 {
	if ((channel < 0) | (channel > 7)) return;			// Make sure we only check legal channel values
	alarm &= ~(0x01 << channel);						// Clear alarm for specified channel
	numFails[channel] = 0;								// Reset fails counter for specified channel
	return;
 }

 /* This function clears all alarms */
 void AFS_clearAlarmAll()
 {
	alarm = 0x00;
	return;
 }

 /* This function returns the average ADC result of the last batch of samples taken for a given channel */
 unsigned int AFS_returnAverageReading(int channel)
 {
 	if ((channel < 0) | (channel > 7)) return 0;		// Make sure we only check legal channel values
	return ADC_averageSamples(channel);
 }

 /* This function takes readings, averages, and sets alarm status for every enabled channel */
 void AFS_monitorChannels()
 {
	int channel;
	for (channel = 0; channel < MAX_NUM_CHANNELS; channel++)
		{
			/* Check if this channel is enabled, else go to the next one */
			if (!(AFS_returnEnabledStatus(channel))) continue;

			/* Select correct channel */
			ADC10CTL0 &= ~ENC;							// Disable conversion to select channel
			ADC10CTL1 &= ~0xF000;						// Clear channel select
			ADC10CTL1 |= (channel << 12);				// Select correct channel
			ADC10CTL0 |= ENC;							// Enable Conversions

			for (sampleIndex = 0; sampleIndex < ADC_NUM_SAMPLES; sampleIndex++)
			{
				/* Take an ADC sample, wait in LPM0 for completion */
				ADC10CTL0 |= ADC10SC;		             					// Trigger a Start of Conversion (ADC10SC)
				__bis_SR_register(CPUOFF + GIE);							// Enter LPM0 while we wait for conversion to finish
				//while(ADC10CTL1 & 0x01);									// While ADC10 is Busy (ADC10BUSY flag)

				lastResult = ADC10MEM;

				/* Return from interrupt and store conversion result */
				ADCValue[channel][sampleIndex] = lastResult;


				/* Send result over UART to PC for the GUI */
				UART_resultToGUI(lastResult, channel);
			}

			/* Average the ADC samples */
			sampleAverage[channel] = ADC_averageSamples(channel);

			/* Compare ADC Peaks to ADC Average */
			if (sampleAverage[channel] < threshold)								// If the average is below the failing threshold
			{
				numFails[channel]++;											// If peaks < 1% greater than the average, there is most likely insufficient airflow, record a failure
				if (numFails[channel] >= MAX_FAILS)								// After a certain number of consecutive failures, raise a flag (here an LED)
				{
					if (AFS_returnEnabledAlarmStatus(channel))					// Check if alarm is enabled for this channel
					{
						alarm |= (0x01 << channel);								// Raise alarm
					}
				}
			}
			else
			{
				numFails[channel] = 0;											// Reset numFails counter after a non-fail
			}

		}
	return;
 }

 /* helper function for averaging */
 unsigned int ADC_averageSamples(int channel)
 {
	unsigned int i, total = 0;

	for(i=0; i < ADC_NUM_SAMPLES; i++)
	{
		total += ADCValue[channel][i];					// Take a running total of all results
	}

	return (total / ADC_NUM_SAMPLES);					// Return the average of the results
 }

 /* this function sends the most recent sample over UART to the PC side GUI */
 void UART_resultToGUI(unsigned int result, int channel)
 {
	uart_putc((char)result);						// 8 LSBs of result
	uart_putc((char)(result >> 8));					// 2 MSBs of result

	return;
 }



 //
 // Interrupt Service Routines
 //
 #pragma vector=ADC10_VECTOR
 __interrupt void ADC10_ISR(void)			// ADC10 interrupt service routine
 {
 	__bic_SR_register_on_exit(LPM0_bits);   // Return from LPM0
 }
